package com.taais.biz.controller;


import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.domain.DataAugmentation;
import com.taais.biz.domain.bo.DataAugmentationBo;
import com.taais.biz.domain.bo.VideoStableStartResultBo;
import com.taais.biz.domain.vo.DataAugmentationVo;
import com.taais.biz.service.IVideoStableService;
import com.taais.common.core.service.OssService;
import com.taais.system.domain.SysOss;
import com.taais.system.service.ISysOssService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import jakarta.annotation.Resource;
import com.taais.biz.service.IDataAugmentationService;

import com.taais.common.core.core.page.PageResult;

/**
 * 数据增强Controller
 *
 * @author 0
 * 2024-08-30
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/dataAugmentation")
public class DataAugmentationController extends BaseController {
    @Resource
    private IDataAugmentationService dataAugmentationService;
    @Autowired
    private ISysOssService ossService;
    @Autowired
    private IVideoStableService videoStableService;
    @GetMapping("/compare/num/{task_id}")

    public CommonResult getCompareNum(@PathVariable("task_id") Long taskId) {
        return CommonResult.success(dataAugmentationService.getCompareNum(taskId));
    }


    private static Path getImageAtPathIdx(Path inputPath, int idx) throws IOException {
        try (Stream<Path> paths = Files.list(inputPath)) {
            return paths.sorted()
                .skip(idx)
                .findFirst()
                .orElseThrow(() -> new IllegalArgumentException("Index " + idx + " is out of bounds for the list of files in " + inputPath));
        }
    }

    @GetMapping("/compare/{task_id}/{idx}")
    public ResponseEntity<Map<String,List<String>>> getCompareImages(@PathVariable("task_id") Long taskId, @PathVariable("idx") int idx) {
        try {
            DataAugmentation dataAugmentation = dataAugmentationService.getById(taskId);

            Path inputPath = null;
            Path outputPath = null;
            String osName = System.getProperty("os.name");
            // 判断是否是Windows环境
//            if (osName.toLowerCase().contains("windows")) {
//                inputPath = Paths.get("D:", dataAugmentation.getInputPath());
//                outputPath = Paths.get("D:", dataAugmentation.getOutputPath());
//            } else {
//                inputPath = Paths.get(dataAugmentation.getInputPath());
//                outputPath = Paths.get(dataAugmentation.getOutputPath());
//            }
            inputPath = Paths.get(dataAugmentation.getInputPath());
            outputPath = Paths.get(dataAugmentation.getOutputPath());
            System.out.println("inputPath: " + inputPath.toString());
            System.out.println("outputPath: " + outputPath.toString());
            Path imagePath = getImageAtPathIdx(inputPath, idx);

            String fileName = imagePath.getFileName().toString();
            Map<String, List<String>> images = new HashMap<>();
            //图像拼接算法有多个输入图片
            if ("图像拼接_sift".equals(dataAugmentation.getTaskType()) || "图像拼接_coordinate".equals(dataAugmentation.getTaskType())) {
                String lastDirectoryName = imagePath.getFileName().toString();
                outputPath = outputPath.resolve(lastDirectoryName);  //得到推理结果目录
                Stream<Path> stream = Files.list(outputPath);
                if (Files.exists(outputPath) && Files.isDirectory(outputPath)) {
                    // 检查流中是否有下一个元素
                    Path firstRegularFile = stream.filter(Files::isRegularFile).findFirst().orElse(null);
                    if (firstRegularFile != null) {
                        // 找到第一个常规文件
                            fileName = firstRegularFile.getFileName().toString();
                            System.out.println("找到的第一个常规文件: " + fileName);


                    } else {
                        System.out.println("图像拼接算法未在结果目录下保存结果文件！");
                        return ResponseEntity.status(500).build();
                    }
                } else {
                    System.out.println("图像拼接算法未创建结果目录！");
                    return ResponseEntity.status(500).build();
                }
                // 收集所有文件的路径，并编码为Base64字符串
                Stream<Path> newPathStream = Files.list(imagePath);
                List<String> origin = newPathStream
                    .filter(Files::isRegularFile)  // 只选择常规文件（排除子目录）
                    .filter(path -> !path.toString().toLowerCase().endsWith(".txt")).map(path -> {
                        try {
                            return Base64.getEncoder().encodeToString(Files.readAllBytes(path));
                        } catch (IOException e) {
                            throw new RuntimeException(e);
                        }
                    })
                    .collect(Collectors.toList());
                images.put("origin", origin);
            } else {
                byte[] image1 = Files.readAllBytes(imagePath);
                String base64Image1 = Base64.getEncoder().encodeToString(image1);
                ArrayList<String> origin = new ArrayList<>();
                origin.add(base64Image1);
                images.put("origin", origin);
            }
            Path resolve = outputPath.resolve(fileName);
            if (!Files.exists(resolve)) {
                System.out.println("结果文件不存在:" + resolve.toString());
                return ResponseEntity.status(500).build();
            }
            byte[] image2 = Files.readAllBytes(resolve);
            String base64Image2 = Base64.getEncoder().encodeToString(image2);   // 将图片编码成Base64字符串
            ArrayList<String> stable = new ArrayList<>();
            stable.add(base64Image2);
            images.put("stable", stable);
            // 返回Map
            return ResponseEntity.ok(images);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.status(500).build();
        }
    }

    @PostMapping("/get_result")
    public CommonResult getResult(@Valid @RequestBody VideoStableStartResultBo videoStableStartResultBo) {
        return dataAugmentationService.getResult(videoStableStartResultBo);
    }

    @GetMapping("/start/{id}")
    public CommonResult start(@PathVariable("id") Long id) {
        return dataAugmentationService.start(id);
    }

    @GetMapping("/stop/{id}")
    public CommonResult stop(@PathVariable("id") Long id) {
        return dataAugmentationService.stop(id);
    }

    /**
     * 查询数据增强列表
     */
    @SaCheckPermission("demo:dataAugmentation:list")
    @GetMapping("/list")
    public CommonResult<PageResult<DataAugmentationVo>> list(DataAugmentationBo dataAugmentationBo) {
        return CommonResult.success(dataAugmentationService.selectPage(dataAugmentationBo));
    }

    /**
     * 导出数据增强列表
     */
    @SaCheckPermission("demo:dataAugmentation:export")
    @Log(title = "数据增强", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataAugmentationBo dataAugmentationBo) {
        List<DataAugmentationVo> list = dataAugmentationService.selectList(dataAugmentationBo);
        ExcelUtil.exportExcel(list, "数据增强", DataAugmentationVo.class, response);
    }

    /**
     * 获取数据增强详细信息
     */
    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/{id}")
    public CommonResult<DataAugmentationVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(dataAugmentationService.selectById(id));
    }

    /**
     * 新增数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:add")
    @Log(title = "数据增强", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody DataAugmentationBo dataAugmentationBo) {
//        QueryWrapper queryWrapper = new QueryWrapper();
//        queryWrapper.eq("file_name", dataAugmentationBo.getUploadPath());
//        List<SysOss> list = ossService.list(queryWrapper);
//        SysOss oneByEntityId = list.get(0);
//        dataAugmentationBo.setInputOssId(oneByEntityId.getOssId());
        boolean inserted = dataAugmentationService.insert(dataAugmentationBo);
        if (!inserted) {
            return CommonResult.fail("新增数据增强记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:edit")
    @Log(title = "数据增强", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody DataAugmentationBo dataAugmentationBo) {
        Boolean updated = dataAugmentationService.update(dataAugmentationBo);
        if (!updated) {
            return CommonResult.fail("修改数据增强记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:remove")
    @Log(title = "数据增强", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = dataAugmentationService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除数据增强记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 获取日志字符串
     */
    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/log/{id}")
    public CommonResult<String> getLog(@PathVariable Long id) {
        DataAugmentation byId = dataAugmentationService.getById(id);
        String filePath = byId.getAlgorithmPath();
        try{
            Path path = Paths.get(filePath);
            if (!Files.exists(path)) {
                return CommonResult.fail( "日志文件" + filePath + "不存在！");
            }
            // 读取文件内容
            return CommonResult.success(new String(Files.readAllBytes(path)), "获取日志成功");
        } catch (Exception e) {
            return CommonResult.fail("日志文件" + filePath + "读取失败！");
        }

    }
}
